#!/usr/bin/env python
"""
_GetValidation_

Oracle implementation of Subscription.GetValidation
"""

from WMCore.WMBS.MySQL.Subscriptions.GetValidation import GetValidation as GetValidationMySQL

class GetValidation(GetValidationMySQL):
    pass
